
local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Main prefab nodes
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_RootPart", "STONE_GATE_V5_SST_ROOT_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_1_Sst_Tiling1Part", "STONE_GATE_V5_SST_TILING1_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_2_Sst_Tiling2Part", "STONE_GATE_V5_SST_TILING2_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", "STONE_GATE_V5_SST_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.05 }
	    }
	})
end

local defaultNodePrefixList = {
	"STONE_GATE_V5_SST_TOP",
	"STONE_GATE_V5_SST_TILING1",
	"STONE_GATE_V5_SST_TILING2"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the stone gate set small squared tower root part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_SST_ROOT_PART",
	Name = "STONE_GATE_V5_SST_ROOT_NAME",
	Description = "STONE_GATE_V5_SST_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "STONE_GATE_V5_SST_ROOT_PREFAB",
		EndPart = "STONE_GATE_V5_SST_TOP_PART",
		FillerList = {
			"STONE_GATE_V5_SST_TILING2_PART"
                },
		BasementFillerList = {
			"STONE_GATE_V5_SST_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 1 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.5 }
	},
        VillagerCapacity = 3,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV5:configurePrefabFlagList("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_3_Sst_TopPart", { "PLATFORM" })
 
-- Register grass clearing components

StoneGateV5:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_RootPart", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 4 }
})

-- Override building part components properties (generated by the building asset processor) to add building zones
StoneGateV5:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_0_Sst_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 4, 4 }
})

function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", "STONE_GATE_V5_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_SST_" .. _nameAsset .. "_PART",
	Name = "STONE_GATE_V5_SST_" .. _nameAsset .. "_NAME",
	Description = "STONE_GATE_V5_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "STONE_GATE_V5_SST_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 3 }
		}
	}
    })

--    StoneGateV5:registerPrefabComponent("models/basic/basicSmallSquaredTower.fbx/Prefab/smallSquaredTower/Sst_" .. _nameFbx .. "Part", {
--	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
--	BuildingZone = { 4, 4 }
--    })
end

registerRoofPart( "RoofA", "ROOF_A" )
registerRoofPart( "RoofB", "ROOF_B" )

---- Ramparts...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/ramparts/rampartSmallSquaredTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", "STONE_GATE_V5_SST_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_SST_" .. _nameAsset .. "_PART",
	Name = "STONE_GATE_V5_SST_" .. _nameAsset .. "_NAME",
	Description = "STONE_GATE_V5_SST_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "STONE_GATE_V5_SST_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

--    StoneGateV5:registerPrefabComponent("models/ramparts/rampartSmallSquaredTower.fbx/Prefab/smallSquaredTower/step_4_Sst_" .. _nameFbx .. "Part", {
--	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
--	BuildingZone = { 4, 4 }
--    })
end

registerRampartPart( "RampartA", "RAMPART_A" )
registerRampartPart( "RampartB", "RAMPART_B" )
registerRampartPart( "RampartC", "RAMPART_C" )
